function [ eval, isterminal, dir ] = TPM_events( N, coeffs, model, t, Y )
% Copyright Michael Sargent 2015
% 
% TPM_EVENTS is used to cease the solution of the system of differential
% equations specified by TPM_gradient when one of a number of different
% conditions is reached. These three events are resweeping, steady state
% achieved, and bubble collapse. 
% 
% [ eval, isterminal, dir ] = TPM_events( N, coeffs, model, t, Y )
% 
%
%   INPUTS
% 
%   N           scalar                  Number of concentration points.
% 
%   coeffs      structure               Contains all the material
%                                       properties of the fluid and gas,
%                                       and the bulk conditions of
%                                       temperature and pressure, and the
%                                       boundary and initial conditions of
%                                       the system. 
%
%   t           scalar                  Current time, for evaluating the
%                                       event functions. 
%
%   Y           column vector (3N+3,1)  Contains all the R, r(i), PB, C(i),
%                                       TB and T(i) data in that order, for
%                                       N concentration points.
% 
% 
%   OUTPUTS
%
%   eval        vector  (3)             Results of the three event
%                                       functions. When one of these values
%                                       is zero, the ode solver regards
%                                       this as an event.
%
%   isterminal  vector  (3)             Value = 1 if the solver is to
%                                       terminate at a given event, = 0
%                                       otherwise. 
% 
%   dir         vector  (3)             Value = 0 if all zeros are counted
%                                       as events, +1 if only zeros where
%                                       eventvalue is increasing, and -1 if
%                                       only zeros where the eventvalue is 
%                                       decreasing.
% 
% 
% There are a number of events which affect the system:
% 
% Firstly, if the ratio between the largest and smallest raidal gaps
% becomes too great then the radial positions need to be resweeped and the 
% concentrations and temperatures must be interpolated at the new
% positions. 
% 
% Secondly, if all the time derivatives are close enough to zero then the 
% system is at steady state and so computation can cease.
% 
% Lastly, if the bubble radius decreases below a tiny value then the bubble
% has collapsed to nothingness because the initial conditions were badly
% defined.
%

%%
% *************************************************************************
% Resweeping event
% *************************************************************************

% The two extremes of interval width are at the bubble interface and at the
% influence cell. The event is triggered when the ratio of the largest to
% smallest becomes more than a given threshold.
intervals = [(Y(N+1)-Y(N)),(Y(2)-Y(1))];
max_ratio = max(intervals) / min(intervals);
threshold_ratio = 2;
eval(1) = max_ratio - threshold_ratio;

% Terminate evaluation at this event.
isterminal(1) = 1;

% This event triggers when the ratio increases above the threshold, so
% is increasing in value of eventvalue(3).
dir(1) = +1;


%%
% *************************************************************************
% Steady state event
% *************************************************************************

% The time derivatives of all variables are calculated, and made fractional
% by dividing by the original variables. The event is triggered when the
% root mean square of these residuals becomes smaller than a given
% threshold. 
residuals = TPM_gradient(N,coeffs,model,t,Y) ./ Y;
rms_residual = rms(residuals);
threshold_residual = 10^-6;
eval(2) = rms_residual - threshold_residual;

% Terminate evaluation at this event.
isterminal(2) = 1; 

% This event triggers when the derivative decreases below the threshold, so
% is decreasing in value of eventvalue(2).
dir(2) = -1;


%%
% *************************************************************************
% Bubble collapse event
% *************************************************************************

% The event is triggered when the radius becomes smaller than a given
% threshold. 
bubble_radius = Y(1);
threshold_radius = 10^-8;    % 10 nanometers, approximate molecular size.
eval(3) = bubble_radius - threshold_radius;

% Terminate evaluation at this event.
isterminal(3) = 1;

% Trigger event in both directions.
dir(3) = 0;


end