function [ dY_dt ] = TPM_gradient( N, coeffs, model, t, Y )
% Copyright Michael Sargent 2015
% 
% TPM_gradient( t, Y ) is for use in solving the combined total and partial
% derivatives that describe the radius, concentration, and temperature of a
% generalised Newtonian fluid surrounding a fluid bubble that is growing or
% shrinking due to thermodynamic driving forces. 
%
% Using the current status of the system, this function calculates the rate
% of change of each of the variables in Y with respect to time t. It does
% so without iterating by use of element-wise vector operations, so
% speeding up the process.
%
% [ dY_dt ] = TPM_gradient( N, coeffs, model, t, Y ) 
% 
%
%   INPUTS
%
%   N       scalar                  Number of concentration points used.
% 
%   coeffs  structure               Contains all the material properties of
%                                   the fluid and gas, and the bulk
%                                   conditions of temperature and pressure,
%                                   and the boundary and initial conditions
%                                   of the system.
%
%   model   fluid_model             A custom class which contains all the
%                                   data about the fluid model used. It
%                                   includes equations for viscosity,
%                                   viscous dissipation, and bubble growth,
%                                   along with the necessary parameters of
%                                   that model. 
%
%   t       scalar      (1)         Current time, for evaluating the
%          /row vector  (1,L)       boundary condition functions. Can be a
%                                   single time or a vector of different
%                                   times.
%
%   Y       vector      (3N+3,1)    Contains all the R, r(i), PB, C(i), TB
%          /matrix      (3N+3,L)    and T(i) data in that order, for N 
%                                   concentration points. Calculations are
%                                   'vectorized' to allow a matrix input
%                                   of multiple (L) states simultaneously. 
% 
% 
%   OUTPUTS
%
%   dY_dt   vector      (3N+3,1)    Contains the rate of change with time
%          /matrix      (3N+3,L)    (t) of the corresponding element of Y.
%
%
% The jacobian pattern for this gradient function is constructed by the
% function 'TPM_jpattern.m'
% 

%%
% Check that the value of N matches the size of Y.
if size(Y,1)~=(3*N+3)
    error('Mismatch between supplied value of N and matrix Y size')
end

% Check that t is not a matrix or column vector.
if size(t,1)~=1
    error(['Variable t must not have multiple columns, it must be ' ...
        'scalar or a row vector'])
end

% Check that Y and t use the same value of L for their row length, or
% update appropraitely if possible. 
if size(Y,2)==size(t,2)
    % If they are equal, the value of L is not used.
    L = size(Y,2);
    
else
    if size(Y,2)==1
        % If Y is a single state but t is not, repeat Y for each t value.
        L = size(t,2);
        Y = repmat(Y, 1, L);
        
    elseif size(t,2)==1        
        % If t is a single value but Y is not, repeat t for each Y state.
        L = size(Y,2);
        t = repmat(t, 1, L);
        
    else
        % If the L values of Y and t are not the same, but neither are 1
        % then there is an irreconcilable mismatch.
        error('Mismatch between vectorised row lengths (L) of Y and t')
    end
    
end

if min(Y)<0
    % If any of the positions, concentrations, or temperatures is negative
    % then the state is ill defined. Rather than returning an error, the
    % output of zero is given as this situation may be reached accidentally
    % while root finding or attempting implicit solution of the ODEs.
    dY_dt = 0;
    return
end


%%
% Unpack the useful data into separate vectors.

% Bubble radius, first point radius, cell radius, second to last radius,
% and radii vector. 
R   = Y(1,:);               % 1 by L
r1  = Y(2,:);               % 1 by L
S   = Y(N+1,:);             % 1 by L
S1  = Y(N,:);               % 1 by L

ri  = Y(2:N+1,:);           % N by L

% Bubble pressure, first point concentration, bulk concentration, and 
% concentration vector.
PB  = Y(N+2,:);             % 1 by L
C1  = Y(N+3,:);             % 1 by L
CS  = Y(2*N+2,:);           % 1 by L

Ci  = Y(N+3:2*N+2,:);       % N by L

% Bubble temperature, first point temperature, and temperature vector.
TB  = Y(2*N+3,:);           % 1 by L
T1  = Y(2*N+4,:);           % 1 by L

Ti  = Y(2*N+4:3*N+3,:);     % N by L

% Evaluate the boundary condition functions for the current time.
Tf  = coeffs.Tf(t);         % 1 by L
Pf  = coeffs.Pf(t);         % 1 by L


%%
% The bubble growth rate depends on the modified pressure driving force and
% the average temperature of the system.
delta_P_star = sqrt(3) / 2 * ( PB - Pf - 2 * coeffs.sigma ./ R );
T_av = (TB + Tf) / 2;

% The correct bubble growth equation is supplied as a method in the
% fluid_model class, and always uses the same syntax regardless of model.
dR_dt = bubblegrowth(model, delta_P_star, R, S, T_av);

% The class also supplies an equation for the viscous dissipation, which
% requires an expression for the strain rate magnitude at all the points.
gamma_dot_i = sqrt(12) * repmat(R.^2 .* abs(dR_dt), N, 1) ./ ri.^(3);
phi_i = dissipation(model, gamma_dot_i, Ti);

%%
% Rate of change of bubble temperature.
dTB_dt = 3 * coeffs.Rgas / coeffs.mR / coeffs.CPg * TB ./ R ./ PB .* ( ...
    coeffs.k * ( T1 - TB ) ./ ( r1 - R ) ...
    - coeffs.D * coeffs.Hvap * ( C1 - PB * coeffs.KH ) ./ (r1 - R) ...
    - 2 * coeffs.sigma * dR_dt ./ R );


%%
% Rate of change of bubble pressure (dependent on rate of change of bubble
% temperature).
dPB_dt = 3 * coeffs.Rgas * coeffs.D / coeffs.mR * TB ./ R .* ...
    ( C1 - PB * coeffs.KH ) ./ (r1 - R) ...
    - 3 * PB .* dR_dt ./ R ...
    + PB ./ TB .* dTB_dt;


%%
% Create the shifted vectors that allow the discretised partial derivatives
% to be calculated without iterating through the elements of Ci, ri and Ti.
% These have the property that:
% Xi_plus1(i,:) = Xi(i+1,:)
% Xi_minus1(i,:) = Xi(i-1,:)
% So that the equations are reduced to elementwise vector operations. The
% boundary conditions are incorporated into Xi_plus1(N,:) and 
% Xi_minus1(1,:).

ri_minus1 = [ R             ; ri(1:N-1,:)   ];  % ri(0) = R
ri_plus1 =  [ ri(2:N,:)     ; (2*S - S1)    ];  % equal spacing

Ci_minus1 = [ PB*coeffs.KH  ; Ci(1:N-1,:)   ];  % Ci(0) = PB * KH
Ci_plus1 =  [ Ci(2:N,:)     ; CS            ];  % Ci(N+1) = Ci(N)

Ti_minus1 = [ TB            ; Ti(1:N-1,:)   ];  % Ti(0) = TB
Ti_plus1 =  [ Ti(2:N,:)     ; Tf            ];  % Ti(N+1) = Tf


%%
% Rate of change of positions.
dri_dt = repmat(R.^2 .* dR_dt, N, 1) ./ ri.^(2) ;


%%
% Rate of change of concentrations.
dCi_dt = coeffs.D / 2 ./ ri.^2 ./ ( ri_plus1 - ri_minus1) .* ...
    ( ( ri_plus1 + ri ).^2 .* ( Ci_plus1 - Ci ) ./ ( ri_plus1 - ri ) - ...
    ( ri + ri_minus1 ).^2 .* ( Ci - Ci_minus1 ) ./ ( ri - ri_minus1 ) );


%%
% Rate of change of temperatures, uses viscous dissipation.
dTi_dt = coeffs.k / 2 / coeffs.rho / coeffs.CPf ./ ri.^2 ./ ...
    ( ri_plus1 - ri_minus1) .* ( ...
    ( ri_plus1 + ri ).^2 .* ( Ti_plus1 - Ti ) ./ ( ri_plus1 - ri ) - ...
    ( ri + ri_minus1 ).^2 .* ( Ti - Ti_minus1 ) ./ ( ri - ri_minus1 ) ) ...
    + phi_i / coeffs.rho / coeffs.CPf;


%%
% Concantenate the vector of time derivatives back together:
dY_dt = [dR_dt; dri_dt; dPB_dt; dCi_dt; dTB_dt; dTi_dt];


end