function [ Jpattern ] = TPM_jpattern( N )
% Copyright Michael Sargent 2015
%
% TPM_JPATTERN returns the jacobian pattern of the function
% "TPM_gradient.m" which is used to determine the rate of change of all the
% positions, concentrations, and temperatures of a number of fluid elements
% surrounding a growing or shrinking gas bubble in a generalised Newtonian
% fluid. 
%
% Jpattern(i,j) = 1 if dY_dt(i) depends on Y(j), and 0 if it does not.
% 
% [ Jpattern ] = TPM_jpattern( N )
% 
%
%   INPUTS
% 
%   N           scalar                      Number of concentration points,
%                                           must be a positive nonzero
%                                           integer.
%
% 
%   OUTPUTS
%
%   Jpattern    sparse matrix   (3N+3,3N+3) Jacobian pattern for speeding 
%                                           up an ode solver.
%
%
% The sparse pattern is constructed without iterating by use of sparse
% diagonalisation and row filling. The constructed matrix has exactly
% 22N+7 non-zero elements so has a sparsity that tends toward 22/N for
% sufficiently large N.
% 
% Split the vector Y into thirds: r, C and T:
%   r includes the bubble radius R and all point radii ri
%   C includes the bubble pressure PG and all point concentrations Ci
%   T includes the bubble temperature TG and all point temperatures Ti
% 
% Form the jacobian out of 9 smaller matrices, one for how each third
% depends on each other third:
% 
%  |----------------------|
%  | J_rr  , J_rC  , J_rT |
%  | J_Cr  , J_CC  , J_CT |
%  | J_Tr  , J_TC  , J_TT |
%  |----------------------|
%
% The total maxtrix is (3N+3,3N+3) and each part-matrix is (N+1,N+1).

%%
% Check it is possible.
if N == 0
    error('N cannot be 0')
end

%%
% Special case for (highly inaccurate) film model
if N == 1
    Jpattern = sparse(ones(6,6));
    Jpattern = Jpattern - sparse(...
        [1,2,6,4,1,2,4],...
        [4,4,4,5,6,6,6],...
        [1,1,1,1,1,1,1],...
        6,6,7);
    return
end

%%
% *************************************************************************
% J_rr
% *************************************************************************
% Contains (N+1) + (N+1) + (N+1) elements, with 2 overlaps at (1,1) and 
% (N+1,N+1); therefore contains (3N+1) elements.

% All dri_dt depend on R, the first element of the ri vector, and on S
% through the dependency on Rdot.
J_rr = sparse(...
    [1:N+1 1:N+1],...                   % i coordinates
    [ones(N+1,1) (N+1)*ones(N+1,1)],... % j coordinates
    ones(2*N+2,1),...                   % values
    N+1,N+1,...                         % matrix size
    3*N+1);                             % maximum number of elements

% Also all dri_dt depend on themselves, the main (0th) diagonal of the 
% matrix.
J_rr = spdiags(...
    ones(N+1,1),...                     % values
    0,...                               % main diagonal
    J_rr);                              % add to J_rr


%%
% *************************************************************************
% J_rC
% *************************************************************************
% Contains exactly (N+1) elements.

% All dri_dt depend only on PB, the first element of Ci vector, through the
% dependency on Rdot.
J_rC = sparse(...
    1:N+1,...                           % i coordinates
    ones(N+1,1),...                     % j coordinates
    ones(N+1,1),...                     % values
    N+1,N+1,...                         % matrix size
    N+1);                               % maximum number of elements


%%
% *************************************************************************
% J_rT
% *************************************************************************
% Contains exactly (N+1) elements.

% All dri_dt depend only on TB, the first element of Ti vector, through the
% dependency on Rdot.
J_rT = sparse(...
    1:N+1,...                           % i coordinates
    ones(N+1,1),...                     % j coordinates
    ones(N+1,1),...                     % values
    N+1,N+1,...                         % matrix size
    N+1);                               % maximum number of elements


%%
% *************************************************************************
% J_Cr
% *************************************************************************
% Contains exactly (3N+1) + (1) elements, with no overlaps (unless N==1);
% therfore contains (3N+2) elements.

% Create an empty sparse matrix of the right size
J_Cr = sparse(...
    [],...                              % i coordinates
    [],...                              % j coordinates
    [],...                              % values
    N+1,N+1,...                         % matrix size
    3*N+2);                             % maximum number of elements

% All dC_dt(i) depend on r(i-1), r(i), and r(i+1), which are diagonals
% numbered -1, 0, and 1. Exceptions are dC_dy(i=0) which does not depend on
% r(i=-1) and dC_dt(i=N+1) which does not depend on r(i=N+2). This is
% accounted for in the sparse diagonalisation operation.
J_Cr = spdiags(...
    ones(N+1,3),...                     % values, three columns of ones
    [-1 0 1],...                        % three major diagonals
    J_Cr);                              % add to J_Cr

% Finally, dPB_dt also depends on S through Rdot. 
J_Cr(1,N+1) = 1;


%%
% *************************************************************************
% J_CC
% *************************************************************************
% Contains exactly (3N+1) elements.

% Depends on the three diagonals of concentration, start with an empty
% sparse matrix of the right size:
J_CC = sparse(...
    [],...                              % i coordinates
    [],...                              % j coordinates
    [],...                              % values
    N+1,N+1,...                         % matrix size
    3*N+1);                             % maximum number of elements

% All dC_dt(i) depend on C(i-1), C(i), and C(i+1), which are diagonals
% numbered -1, 0, and 1. Exceptions are dC_dt(i=0) which does not depend on
% C(i=-1) and dC_dt(i=N+1) which does not depend on C(i=N+2). This is
% accounted for in the sparse diagonalisation operation.
J_CC = spdiags(...
    ones(N+1,3),...                     % values, three columns of ones
    [-1 0 1],...                        % three major diagonals
    J_CC);                              % add to J_CC


%%
% *************************************************************************
% J_CT
% *************************************************************************
% Contains exactly (2) elements.

% Only dPB_dt has temperature dependence, it may depend on TG and T1, which
% are the first and second elements of the Ti vector.
J_CT = sparse(...
    [1 1],...                           % i coordinates
    [1 2],...                           % j coordinates
    [1 1],...                           % values
    N+1,N+1,...                         % matrix size
    2);                                 % maximum number of elements


%%
% *************************************************************************
% J_Tr
% *************************************************************************
% Contains (3N+1) + (N+1) + (N+1) + (N+1) elements with 7 overlaps at
% (1,1), (2,1), (1,2), (2,2), (3,2), (N,N+1), and (N+1,N+1); therefore
% contains (6N-3) elements. 

% All dTi_dt depend on R, r1, and S, through viscous heating:
J_Tr = sparse(...
    [1:N+1 1:N+1 1:N+1],...             % i coordinates
    [ones(N+1,1) 2*ones(N+1,1) (N+1)*ones(N+1,1)],...   % j coordinates
    ones(3*N+3,1),...                   % values
    N+1,N+1,...                         % matrix size
    6*N-3);                             % maximum number of elements

% Also all depend on the three diagonals of position:
J_Tr = spdiags(...
    ones(N+1,3),...                     % for each major diagonals 
    [-1 0 1],...                        % three major diagonals
    J_Tr);                              % add to J_Tr


%%
% *************************************************************************
% J_TC
% *************************************************************************
% Contains (N+1) + (1) elements, with no overlaps, so (N+2) in total.

% All dTi_dt depend on PB through Rdot in viscous heating:
J_TC = sparse(...
    1:N+1,...                           % i coordinates
    ones(N+1,1),...                     % j coordinates
    ones(N+1,1),...                     % values
    N+1,N+1,...                         % matrix size
    N+2);                               % maximum number of elements

% In addition, TB depends on C1 via the latent heat transfer at the
% interface:
J_TC(1,2) = 1;


%%
% *************************************************************************
% J_TT
% *************************************************************************
% Contains (N+1) + (3N+1) elements, with 2 overlaps at (1,1) and (1,2);
% therefore contains (4N) elements in total.

% All dTi_dt depend on TB through Rdot in viscous heating:
J_TT = sparse(...
    1:N+1,...                           % i coordinates
    ones(N+1,1),...                     % j coordinates
    ones(N+1,1),...                     % values
    N+1,N+1,...                         % matrix size
    4*N);                               % maximum number of elements

% Also all depend on the three diagonals of temperature:
J_TT = spdiags(...
    ones(N+1,3),...                     % for each major diagonals 
    [-1 0 1],...                        % three major diagonals
    J_TT);                              % add to J_TT


%%
% Join all 9 part jabocians together to give the full jacobian pattern.
% Contains a total of 22N+7 elements unless N==1, when the calculation
% fails.

Jpattern = [J_rr , J_rC , J_rT ; ...
            J_Cr , J_CC , J_CT ; ...
            J_Tr , J_TC , J_TT ];

end