function [results,runtime] = TPM_results( N, coeffs, model, t, Y, savename)
% Copyright Michael Sargent 2015
%
% TPM_RESULTS is used to produce the results data structure from the output
% of a three part model as produced by TPM_solver. It separates the
% positons, concentrations, partial pressure, and temperatures, and
% calculates the strain rate and dimensionless radius for all times and
% positions. It also saves the results automatically to 'savename.mat' if
% the name is unique.
% 
% [ results ] = TPM_results( N, coeffs, model, t, Y, savename )
%
%
%   INPUTS
%
%   N           scalar                  Number of data points used.
% 
%   coeffs      structure               Contains all the material
%                                       properties of the fluid and gas,
%                                       and the bulk conditions of
%                                       temperature and pressure, and the
%                                       boundary and initial conditions of
%                                       the system. 
%
%   model       fluid_model             A custom class which contains all
%                                       the data about the fluid model
%                                       used. It includes equations for
%                                       viscosity, viscous dissipation, and
%                                       bubble growth, along with the
%                                       necessary parameters of that model.
%
%   t           row vector  (1,L)       Times when the matrix Y is
%                                       evaluated. 
%
%   Y           matrix      (3N+3,L)    Matrix of positions, concentrations
%                                       and temperatures at every time
%                                       specified in vector t.
%
%   savename    string                  Name under which the results will
%                                       be automatically saved, if no file
%                                       already exists of that name.
%
%
%   OUTPUTS
%
%   results     structure   -           Contains all the results of the 
%                                       model in more convenient forms. 
%                                       Contains the fields:
% 
%   -R          row vector  (1,L)       Bubble radius at all times
%   -S          row vector  (1,L)       Influence cell radius
%   -PG         row vector  (1,L)       Bubble pressure
%   -ri         matrix      (N+1,L)     All point radii
%   -Ci         matrix      (N+1,L)     All point concentrations
%   -Ti         matrix      (N+1,L)     All point temperatures
%   -R_dot      row vector  (1,L)       Bubble growth rate
%   -PG_dot     row vector  (1,L)       Time derivative of PG
%   -ri_dot     matrix      (N+1,L)     Time derivative of ri
%   -Ci_dot     matrix      (N+1,L)     Time derivative of Ci
%   -Ti_dot     matrix      (N+1,L)     Time derivative of Ti
%   -gamma      matrix      (N+1,L)     Strain rate magnitude 
%   -theta      matrix      (N+1,L)     Dimensionless point radii
%


%%
% Separate the matrix Y by position, concentration and temperature, convert
% the gas pressure to a concentration with KH
results.R = Y(1,:);
results.S = Y(N+1,:);
results.PB = Y(N+2,:);
results.ri = Y(1:N+1,:);
results.Ci = Y(N+2:2*N+2,:);
results.TB = Y(2*N+3,:);
results.Ti = Y(2*N+3:3*N+3,:);

% Convert the gas pressure to equilibrium concentration in Ci
results.Ci(1,:) = results.Ci(1,:) * coeffs.KH;


%%
% Use the gradient function to evaluate the time derivative.
Y_dot = TPM_gradient(N,coeffs,model,t,Y);

results.R_dot = Y_dot(1,:);
results.PB_dot = Y_dot(N+2,:);
results.ri_dot = Y_dot(1:N+1,:);
results.Ci_dot = Y_dot(N+2:2*N+2,:);
results.Ti_dot = Y_dot(2*N+3:3*N+3,:);

% Convert the gas pressure to equilibrium concentration in Ci_dot
results.Ci_dot(1,:) = results.Ci_dot(1,:) * coeffs.KH;


%%
% Strain rate magnitude, for all times and places. Repmat is needed to
% equate vector and matrix sizes.
results.gamma = sqrt(12) * repmat(results.R.^2 .* results.R_dot, N+1, 1)...
    ./ results.ri.^3 ;


%%
% Dimensionless radius, for all times and places. This is a number in the
% interval (0,1) that places the radius between R and S for that given
% time. Repmat is again needed to equate vector and matrix sizes.
results.theta = ( results.ri - repmat(results.R, N+1, 1) ) ./ ...
    repmat( ( results.S - results.R ), N+1, 1);


%%
% Check if a file of that name already exists in the current working
% directory, and save automatically if there is no conflict. If there is a
% conflict, suffix on the date and time to ensure a unique name.
if (exist([pwd '\' savename '.mat'],'file'))
    savename = [savename datestr(clock,'-yyyymmdd-HHMMSS-FFF')];
    message = 'File name not unique, results saved instead as ';
else
    message = 'File saved as ';
end

% save the data
if ~strcmp(savename,'nosave')
    save(savename,'coeffs','model','N','t','Y','results')
    disp([message savename '.mat'])
    disp(['in ' pwd])
else
    disp('output not saved as requested')
end

% finish the timer, display and save the result
toc
runtime = toc;  
if ~strcmp(savename,'nosave')
    save(savename,'runtime','-append')
end

disp(' ')


end