function [ new_Y ] = TPM_resweep( N, coeffs, old_Y, type )
% Copyright Michael Sargent 2015
%
% TPM_RESWEEP is used to rearrange the position, concentration, and
% temperature points within the "TPM_solver.m" function so that the
% radial points don't all end up bunched together around the bubble
% interface. A new distribution ("sweep") of position points is chosen and
% the concentrations and temperatures at those new points are interpolated
% from the old values at the old positions.
% 
% [ new_Y ] = TPM_resweep( N, coeffs, old_Y, type )
%
%
%   INPUTS
%   
%   N       scalar                      Number of concentration points.
% 
%   coeffs  structure                   Contains all the material
%                                       properties of the fluid and gas,
%                                       and the bulk conditions of
%                                       temperature and pressure, and the
%                                       boundary and initial conditions of
%                                       the system. 
%
%   old_Y   column vector   (3N+3,1)    Values of the radius,
%                                       concentration, and temperature for 
%                                       all points, needing the positions
%                                       to be changed to a more even
%                                       spacing.  
%
%   type    string                      Selects the distribution of the new
%                                       radii, 'linear', 'squared' or
%                                       'cubic'. The default is 'linear'.
% 
% 
%   OUTPUTS
%
%   new_Y   column vector   (3N+3,1)    New state vector.
%
%
% The distribution of the new points is calculated using the 'TPM_sweep.m'
% function.
%

%%
% Separate Y into different sets for r, C and T.
old_ri = old_Y(1:N+1);
old_Ci = old_Y(N+2:2*N+2);
old_Ti = old_Y(2*N+3:3*N+3);

% Turn the gas pressure into a concentration for interpolation purposes.
old_Ci(1) = old_Ci(1) * coeffs.KH; 


%%
% Generate the new distribution of positions with the sweep function. 
new_ri = TPM_sweep( N, old_ri(1), old_ri(end), type);

% Interpolate the old radii and values into the values at the new radii.
new_Ci = interp1(old_ri,old_Ci,new_ri);
new_Ti = interp1(old_ri,old_Ti,new_ri);

% Turn the equilibrium concentration back into a pressure.
new_Ci(1) = new_Ci(1) / coeffs.KH; 


%%
% Concantenate the three sets into the single column vector output
new_Y = [ new_ri ; new_Ci ; new_Ti ];


end