function [ N, coeffs, model, savename ] = TPM_setup( savenamein ) 
% Copyright Michael Sargent 2015
%
% TPM_SETUP is used to set up the three part model of a bubble growing in a
% finite medium of fluid. It defines three global variables which are
% subsequently used by many of the functions in the TPM problem solver. The
% default values are those for an instant coffee/nitrogen system. 
% 
% [ N, coeffs, model, savename ] = TPM_setup( savenamein )
% 
%
%   INPUTS
%
%   savenamein      string      Sets the file name under which to save this
%                               test as 'savenamein.mat' in the current
%                               working directory.
%
%
%   OUTPUTS
%
%   N               scalar      The number of data points with which to
%                               discretise the space.
%
%   coeffs          stucture    A structure containing the fields:
%
%       -Hvap       scalar      % vaporisation enthalpy % J kg^-1
%       -mR         scalar      % gas molecular mass    % kg kmol^-1
%       -CPg        scalar      % gas heat capacity     % J kg^-1 K^-1
%       -sigma      scalar      % surface tension       % Pa m
%       -D          scalar      % gas diffusivity       % m^2 s^-1
%       -KHmod      scalar      % Henry's constant(mod) % kg m^-3 Pa^-1
%       -ratio      scalar      % Ratio of S0 to R0     % -
%       -KH         scalar      % Henry's constant      % kg m^-3 Pa^-1
%       -k          scalar      % fluid conductivity    % J m^-1 K^-1 s^-1
%       -CPf        scalar      % fluid heat capacity   % J kg^-1 K^-1
%       -rho        scalar      % fluid density         % kg m^-3
%       -Rgas       scalar      % ideal gas constant    % J kmol^-1 K^-1
%       -Tf(t)      handle      % temperature function  % K (s)
%       -Pf(t)      handle      % pressure function     % Pa (s)
%       -R0         scalar      % bubble radius         % m
%       -S0         scalar      % cell radius           % m
%       -C0         scalar      % concentration         % kg m^-3
%
%   model           fluid_model A custom class which contains all the data
%                               about the fluid model used. It includes
%                               equations for viscosity, viscous
%                               dissipation, and bubble growth, along with
%                               the necessary parameters of that model. 
%
%   savename        string      The name under which to subsequently save
%                               the results.
%


%%
% start a timer
tic

%%
% The number of data points with which to discretise the space.
N = 2000;                       % number of points      % -


%%
% All thermodynamic properties, and boundary conditions.

% Gas only properties (air).
coeffs.Hvap =   0;              % vaporisation enthalpy % J kg^-1
coeffs.mR =     28.959;         % gas molecular mass    % kg kmol^-1
coeffs.CPg =    1198;           % gas heat capacity     % J kg^-1 K^-1

% Binary properties.
coeffs.sigma =  0;              % surface tension       % Pa m
coeffs.D =      7*10^-9;        % gas diffusivity       % m^2 s^-1
coeffs.KHmod =  5.47e-06;       % Henry's constant(mod) % kg m^-3 Pa^-1r
coeffs.ratio =  2;              % Ratio of S0 to R0     % -
coeffs.KH =     coeffs.KHmod / (coeffs.ratio^3-1);
                                % Henry's constant      % kg m^-3 Pa^-1

% Fluid only properties.
coeffs.k =      0.28;           % fluid conductivity    % J m^-1 K^-1 s^-1
coeffs.CPf =    380;            % fluid heat capacity   % J kg^-1 K^-1
coeffs.rho =    1426;           % fluid density         % kg m^-3

% Phyiscal constants.
coeffs.Rgas =   8314.5;         % ideal gas constant    % J kmol^-1 K^-1

% Bulk fluidfunctions, must give vectorised results so that the output has
% the same dimensions as the input. 
coeffs.Tf =     @(t) 373 + t * 0;
                                % temperature function  % (s) K
coeffs.Pf =     @(t) 5*10^5 + t * 0;
                                % pressure function     % (s) Pa

% Initial conditions of the system.
coeffs.R0 =     10 * 10^(-6);   % bubble radius         % m
coeffs.S0 =     coeffs.R0 * coeffs.ratio;  
                                % cell radius           % m
coeffs.C0 =     (coeffs.Pf(0)+2*coeffs.sigma/coeffs.R0)*coeffs.KH;
                                % concentration         % kg m^-3
                                
                                
%%
% The model must extend the fluid_model abstract class
model = molten_coffee;


%%
% Set the save name out the same as taken in.
savename = savenamein;

end