function [ t, Y ] = TPM_solver( N, coeffs, model )
% Copyright Michael Sargent 2015
%
% TPM_SOLVER is used to solve the system of ordinary and partial
% differential equations that describe the concentration, radius and
% temperature of a generalised Newtonian fluid surrounding a gas bubble as
% it shrinks or grows due to a thermodynamic driving force. It
% automatically detects problems in the grid spacing for radial points,
% generating a new linear spacing of radii and interpolating the
% concentrations and temperatures at those positions. 
% 
% [ t, Y ] = TPM_solver( N, coeffs, model )
% 
%
%   INPUTS
%
%   N           scalar                  Number of concentration points.
%
%   coeffs      structure               Contains all the physical
%                                       properties and parameters for the
%                                       system. 
%
%   model       fluid_model             A custom class which contains all
%                                       the data about the fluid model
%                                       used. It includes equations for
%                                       viscosity, viscous dissipation, and
%                                       bubble growth, along with the
%                                       necessary parameters of that model.
%
%
%   OUTPUTS
%
%   t           row vector (1,M)        Times of each step used by the
%                                       solver. 
%
%   Y           matrix (3N+3,M)         For each time, gives the values of
%                                       each variable. 
%
%
% The initial conditions of the bubble are that the system is at thermal
% and mechanical equilibrium, but not chemical equilibrium, so the initial
% growth is entirely driven by a concentration gradient between the bubble
% nucleus and the dissolved gas.
% 
% It initially solves the system from t_start for a duration of timeout,
% although an event manager looks for 3 different events that may be
% experienced by the system. 
% 
% If the spacing of the radial points becomes too dense near the bubble
% interface and too sparse at the edge of the influence cell, then the
% solver stops and the radii are redistributed ("resweeping") to avoid
% problems. The solver is then started again and continues for a further
% timeout, rather than ending at the original finish time. 
% 
% Alternately the solver may stop if all the time derivatives become
% smaller than a set criterion which corresponds to the system being at
% steady state. 
% 
% The final end criterion is if the bubble radius becomes too small, which
% is bubble collapse, a complicated phenomenon not investigated further. 
%
% Within an single call of the solver, it works in a number of small time 
% steps. For each time step, TPM_gradient is called to calculate the time 
% derivative of 3N+3 variables, which are N points of fluid concentration,
% radius, and temperature, and the bubble radius, pressure and
% temperature. 
%
% A jacobian pattern is supplied to drastically speed up the computation
% since the solver ode15s is used which has to numerically calculate the
% jacobian to optimise the solution at every time step. Without the pattern
% it would be making a large number of unnecessary calculations.
%
% When it is over, it returns a single vector for all the times, along
% with a matrix of all 3N+3 variables at each time. The times when the
% points are re-swept are therefore counted twice, for before and after the
% resweeping operation. The times, conditions and types of each encountered
% event is also returned.
% 

%%
% Create the first set of initial conditions as a column vector.
initial_condition  = [...
      TPM_sweep(N,coeffs.R0, coeffs.S0, 'linear')   ; ...  %R to ri(N)
      coeffs.Pf(0) + 2 * coeffs.sigma / coeffs.R0   ; ...  %PG, mech. eq.
      coeffs.C0*ones(N,1)                           ; ...  %Ci(1) to Ci(N)
      coeffs.Tf(zeros(N+1,1))                       ];     %TG to Ti(N)
  
% Options for the solver. 
options = odeset(...
    'JPattern',TPM_jpattern(N),...      % Speed up using jacobian pattern
    'Vectorized','on',...               % gradient function is 'vectorised'
    'MaxOrder',5,...                    % Default of 5, maximum possible 5
    'BDF','off',...                     % Default is use NDF not BDF
    'Events',...                        % Checks for resweeping, steady 
    @(t,Y)TPM_events(N,coeffs,model,t,Y)); % state, and bubble collapse.

%%
% Pre allocate the result vector and matrix. The solver runs until an event
% is reached, or it runs out of time, which is one section. The results of
% each section is appended to this vector and matrix as it is completed.
t = [];
Y = [];

%%
% Infinite loop to keep calculating new sections which each last for a
% maximum of 3600 seconds (1 hour) before assuming steady state is reached.
% The first section starts at t = 0. 
t_start = 0;
timeout = 20;

% Line break, then display the time of the start
% disp(' ')
disp('Start solver    : t = 0 s');

while true
    %%
    % Solve using the ode15s solver, for this section's interval. The
    % solver may fail unexpectedly.
    try
    sol = ode15s(@(t,Y)TPM_gradient(N,coeffs,model,t,Y), ...
        [t_start t_start+timeout], ...
        initial_condition, ...
        options);
    catch
        % in the event of solver failure, save the output so far instead of
        % halting. 
        disp(['Solver ERROR    : t = ' num2str(t(end),4) ' s']);
        break
    end
    
    % Add to the results, one section at a time. Then look at the event
    % index to find the cause of the termination.
    t = [t , sol.x];
    Y = [Y , sol.y];
    
    % Event index does not work as part of the structure
    if isempty(sol.ie)
        ind = 0;
    else
        ind = sol.ie(end);
    end
    
    switch ind
        case 1
            % The first type of event is a resweep, requiring reapplication
            % of the solver. 
            disp(['Resweep points  : t = ' num2str(t(end),4) ' s']);
            
            % Resweep the points
            initial_condition = TPM_resweep(N,coeffs,Y(:,end),'linear');
            
            % Update the solver options to start the first guess of the
            % time interval as double that used just before the resweep,
            % and make the new start time one tenth of the old end time.  
            options = odeset(options,...
                'InitialStep',(t(end)-t(end-1))*0.1);
            t_start = t(end);
            
            % Continue the solver after the resweep, by continuing with the
            % 'while true' loop
            continue
            
        case 2
            % The second type of event is a steady state reached, so the
            % solver ends. 
            disp(['Steady state    : t = ' num2str(t(end),4) ' s']);
            break
            
        case 3
            % The third type of event is bubble collapse, so the solver
            % ends. 
            disp(['Bubble collapse : t = ' num2str(t(end),4) ' s']);
            break
            
        otherwise
            % If there isn't an index in the event index vector, then the
            % solver has timed out.
            disp(['Solver timeout  : t = ' num2str(t(end),4) ' s']);
            break
            
    end
    
end

%%
% Display blank line.
disp(' ')

% Display the final bubble size.
disp(['Final bubble radius : ' num2str( Y(1,end)*10^6 , 4 ) ' microns'])

% Display the RMS residual.
rms_residual = rms( TPM_gradient( N, coeffs, model, t(end), Y(:,end) ) ...
    ./ Y(:,end) );
disp(['RMS residual        : ' num2str( rms_residual, 2 ) ' /s'])

%%

% Finished all intervals, and the result has already been constructed from
% the individual results.
disp(' ')

end