function [ points ] = TPM_sweep( N, point_0, point_N, sweep_type )
% Copyright Michael Sargent 2014
%
% TPM_SWEEP produces a vector of N+1 points with values arranged between 
% point_0 and point_N. It is for the purpose of solving a discretised 
% partial differential equation. The spacing of the points is specified by
% sweep_type.
% 
% [ points ] = TPM_sweep( N, point_0, point_N, sweep_type )
% 
%
%   INPUTS
% 
%   N           scalar          The number of points to use, not including
%                               point_0 as that usually obeys a different
%                               differential equation.
%
%   point_0     scalar          The smallest point value.
%
%   point_N     scalar          The largest point value.
%
%   sweep_type  string          String specifying the distribution of the 
%                               points, 'linear', 'squared' or 'cubic'.
% 
% 
%   OUTPUTS
%
%   points      vector  (N+1,1) Column vector of N+1 points.
% 
% 
% The application this was designed for is to solve the system of
% concentration, position and temperature partial differential equations
% surrounding a gas bubble growing or shrinking in a fluid. Therefore, each
% point corresponds to the radius of a spherical surface, and each valid
% sweep method can be explained in relation to those surfaces.
%
% The 'linear' method means each surface is a constant radial distance out 
% from the previous one. This is the default if no string or an invalid
% string is supplied.
%
% The 'squared' method means each surface has a constant amount more
% surface area than the previous one.
% 
% The 'cubic' method means there is the same volume of fluid between all
% adjacent pairs of surfaces.
%

%%
% Indices of points, not including 0 and N, make a column vector.
i=(1:N-1)';

% Choose the type of sweep.
switch sweep_type
    case 'cubic'
        %%
        % Volume between adjacent points (ignoring 4 pi /3).
        delta_V = (point_N^3 - point_0^3) / N;
        % Volume of each sphere (ignoring 4 pi /3).
        volumes = point_0^3 + i * delta_V;
        % Radius is cube root of volume (ignoring 4 pi /3).
        points = (volumes).^(1/3);
        
        
    case 'squared'
        %%
        % Area difference between adjacent points (ignoring 4 pi).
        delta_A = (point_N^2 - point_0^2) / N;
        % Area of each surface (ignoring 4 pi).
        areas = point_0^2 + i * delta_A;
        % Radius is square root of area (ignoring 4 pi).
        points = (areas).^(1/2);
        
        
    otherwise % Default is 'linear'
        %%
        % Radius difference between adjacent points.
        delta_r = (point_N - point_0) / N;
        % Radius of each surface is the points.
        points = point_0 + i * delta_r;
        
end

%%
% Concantenate with points 0 and N
points = [ point_0 ; points ; point_N ];


end