classdef carreauyasuda_fluid < fluid_model
    % Copyright Michael Sargent 2015
    %
    % CARREAUYASUDA_FLUID is a concrete class which extends the fluid_model
    % abstract class, so provides functionality for the CrT model for
    % bubble growth. It describes a Carreau-Yasuda fluid with temperature
    % dependence. 
    % 
    % If the mobility of the fluid particles obeys an Arrhenius type
    % relationship, then the viscoisty is inversely proportional to the
    % mobility. This relationship is discussed in Fogel'son and Likhachev
    % [2001], Technical Physics, vol 46, no 8, pp 1056-1059. 
    %
    % The extension of this to a Carreau-Yasuda fluid is discussed in Shin
    % and Cho [1994], International journal of heat and mass transfer, vol
    % 37, no 1, pp 19-30. This allows the low shear rate viscosity and
    % transition time to vary with temperature, but no other parameters.
    % The transition time is taken as having no temperature dependence
    % here.
    % 
    % The form of the equation means that there is a finite viscosity at
    % infinite temperature, and a temperature at which there is an infinite
    % viscosity. The rate at which one turns into the other is given by an
    % exponent.
    %
    % All temperature dependent models also contain methods to calculate
    % the individual temperature dependent parameters. In this case, only
    % mu depends on temperature so the function is identical to the
    % viscosity method.
    
    %%
    properties (SetAccess=private)
        % Carreau-Yasuda
        name@char = 'Carreau-Yasuda';
        % In LaTeX math format
        constitutive_equation@char = ... 
            ['\eta = \left( A_{\eta} \exp \left( \dfrac{ B_{\eta} }' ...
            '{ T - T_{s} } \right) - ' ...
            '\eta_{\inf} \right) \left( 1 + \left[ \lambda ' ...
            '| \dot{\gamma} | \right]^a \right)^{\frac{ n - 1 }{ a }} ' ...
            '+ \eta_{\inf}'];
    end
    
    %%
    properties
        % Low strain rate viscosity prefactor [Pa s]
        A_eta@double scalar = 3000;
        % Viscosity exponent [K]
        B_eta@double scalar = 50;         
        % High strain rate viscosity [Pa s]
        etaInf@double scalar = 1;
        % Solidification temperature [K]
        T_s@double scalar = 323;
        % Tranision time [s]
        lambda@double scalar = 1;
        % Transition sharpness parameter [-]
        a@double scalar = 2;
        % Thinning parameter [-]
        n@double scalar = 0.5;
    end
    
    %%
    methods
        %%
        function [ obj ] = carreauyasuda_fluid()
            % Uses the default constructor
        end
        
        %%
        function [ obj ] = set.A_eta( obj, value )
            % Restrict A_eta to positive, non-zero values.
            if value <= 0
                error(['Low strain rate viscosity prefactor A_eta ' ...
                    'must be positive and non-zero'])
            else
                obj.A_eta = value;
            end
        end
        
        %%
        function [ obj ] = set.B_eta( obj, value )
            % Restrict B_eta to positive, non-zero values.
            if value <= 0
                error(['Viscosity exponent B_eta must be positive '...
                    'and non-zero'])
            else
                obj.B_eta = value;
            end
        end
        
        %%
        function [ obj ] = set.etaInf( obj, value )
            % Restrict etaInf to positive values.
            if value < 0
                error(['High strain rate viscosity etaInf must be ' ...
                    'positive'])
            else
                obj.etaInf = value;
            end
        end
        
        %%
        function [ obj ] = set.T_s( obj, value )
            % No restrictions on T_s, but will give infinite viscosity for
            % temperatures below T_s.
            obj.T_s = value;
        end
        
        %%
        function [ obj ] = set.lambda( obj, value )
            % Restrict lambda0 to positive, non-zero values.
            if value <= 0
                error(['Transition time lambda0 must be positive ' ...
                    'and non-zero'])
            else
                obj.lambda = value;
            end
        end
        
        %%
        function [ obj ] = set.a( obj, value )
            % Restrict a to positive, non-zero values.
            if value <= 0
                error(['Transition sharpness parameter a must be ' ...
                    'positive and non-zero'])
            else
                obj.a = value;
            end
        end
        
        %%
        function [ obj ] = set.n( obj, value )
            % Restrict n to positive values.
            if value <= 0
                error('Thinning parameter n must be positive and non-zero')
            else
                obj.n = value;
            end
        end
        
        %%
        function [ eta0 ] = eta0( obj, T )
            % Returns low strain rate viscosity for given temperatures.
            %
            % [ eta0 ] = eta0( obj, T )
            %
            %
            % INPUTS
            %
            % obj           carreauyasuda_fluid
            %
            % T             matrix(x,y)     Fluid temperature. (vectorised)
            %
            % OUTPUTS
            %
            % eta0          matrix(x,y)     Low strain rate viscosity.
            %                               (vectorised)
            %
            
            %%
            % Viscosity is infinite if T <= T_s, so put lower bound on T of
            % T_s.
            T = max( T, obj.T_s );
            
            % Return eta0 of same dimensions.
            eta0 = obj.A_eta * ...
                exp( obj.B_eta ./ ( T - obj.T_s ) );
            
            
        end
        
        %%
        function [ eta ] = viscosity( obj, gamma_dot, T )
            % Returns viscosity for given conditions.
            % 
            % [ eta ] = viscosity( obj, gamma_dot, T )
            % 
            % 
            % INPUTS
            %
            % obj           carreauyasuda_fluid
            %
            % gamma_dot     matrix(x,y)     Strain rate of fluid.
            %                               (vectorised)
            %
            % T             matrix(x,y)     Local fluid temperature.
            %                               (vectorised)
            %
            % OUTPUTS
            %
            % eta           matrix(x,y)     Viscosity under those
            %                               conditions. (vectorised)
            %
            
            %%
            % Check inputs are of same dimensions.
            if ~isequal(size(gamma_dot),size(T))
                error('Different sized inputs not allowed')
            end
            
            % Viscosity of the same dimensions.
            eta = ( eta0( obj, T ) - obj.etaInf ) .* ( ...
                1 + ( obj.lambda .* abs(gamma_dot) ).^(obj.a) ...
                ).^( (obj.n - 1) / obj.a ) + obj.etaInf;
            
            
        end
        
        %%
        function phi = dissipation(obj,gamma_dot,T)
            % Returns volumetric viscous dissipation for given conditions.
            % 
            % [ phi ] = dissipation( obj, gamma_dot, T )
            % 
            % 
            % INPUTS
            %
            % obj           carreauyasuda_fluid
            %
            % gamma_dot     matrix(x,y)     Strain rate of fluid.
            %                               (vectorised)
            %
            % T             matrix(x,y)     Local fluid temperature.
            %                               (vectorised)
            %
            % OUTPUTS
            %
            % phi           matrix(x,y)     Volumetric rate of heat
            %                               generation by viscous
            %                               dissipation. (vectorised)
            %
            
            %%
            % Check inputs are of same dimensions.
            if ~isequal(size(gamma_dot),size(T))
                error('Different sized inputs not allowed')
            end
            
            % Return phi of the same dimensions.            
            phi = ( eta0( obj, T ) - obj.etaInf) .* ( ...
                1 + ( obj.lambda .* abs(gamma_dot) ).^(obj.a) ...
                ).^( (obj.n - 1) / obj.a ) .* gamma_dot.^2 + ...
                obj.etaInf .* gamma_dot.^2;
            
            
        end
        
        %%
        function dR_dt = bubblegrowth(obj,delta_P_star,R,S,T)
            % Returns bubble growth rate for given situation.
            % 
            % [ dR_dt ] = bubblegrowth( obj, delta_P_star, R, S, T )
            % 
            % 
            % INPUTS
            %
            % obj           carreauyasuda_fluid
            %
            % deltaPstar    scalar/         Modified pressure difference
            %               vector(1,L)     between bubble and fluid.
            %                               (vectorised)
            %
            % R             scalar/         Radius of the bubble.
            %               vector(1,L)     (vectorised)
            %
            % S             scalar/         Radius of the fluid influence
            %               vector(1,L)     cell. (vectorised)
            %
            % T             scalar/         Average fluid temperature.
            %               vector(1,L)     (vectorised)
            %
            % OUTPUTS
            %
            % dR_dt         scalar/         Rate of bubble growth.
            %               vector(1,L)     (vectorised)
            %
            
            %%
            % Check inputs are all of same dimensions.
            if ~isequal(size(delta_P_star),size(R),size(S),size(T))
                error('Different sized inputs not allowed')
            end
            
            % Find the size of the row vectors.
            L = length(delta_P_star);
            
            % Newtonian case for the first guess, where mu = eta0. This
            % will always underpredict the growth rate, as it overpredicts
            % the viscosity.
            dR_dt = R .* delta_P_star / sqrt(12) ./ eta0( obj, T );
            
            
            %%
            % Prespecify groups of the parameters before the iteration to
            % speed up calculation, since calling the parameters from the
            % class is time consuming. It also allows as much as possible
            % to be vectorised.
            
            % Used to convert the bubble growth rate into the 
            % nondimensional strain rate magnitude at the bubble interface,
            % without nesting handles.
            dimgroup = sqrt(12) * obj.lambda ./ R;
            
            % Gemoetric relationship between the strain rate magnitudes at
            % the interface and influence cell.
            geometry = ( R ./ S ).^3;
            
            % LHS is the Newtonian result.
            LHS = dR_dt;
            
            % The ratio of low and high strain rate viscosities, used a
            % lot.
            eta_ratio = obj.etaInf ./ eta0( obj, T );
            
            % These two parameters are used in the viscosity term, RHS3
            % below.
            apower = obj.a;
            viscositypower = ( (obj.n - 1) / obj.a );
            
            %%
            % For a Carreau-Yasuda fluid the function in implicit, which
            % means that root finding is necessary and vectorised
            % calculation is impossible. Therefore the L different cases
            % must be iterated through manually.
            for i=1:L
                %%
                % The function used for root finding has 5 terms; one on
                % the left hand side which is independent of dR_dt and 4 on
                % the right hand side which are dependend on dR_dt.
                %
                % RHS2 and RHS4 use Gauss' hypergeometric function, CY_2F1,
                % which is provided in the specific case used for a
                % Carreau-Yasuda fluid by a private method in this class. 
                %
                                
                RHS1 = @(Rdot) Rdot * eta_ratio(i);
                
                RHS2 = @(Rdot) Rdot * (1 - eta_ratio(i)) * ...
                    CY_2F1( obj, dimgroup(i) * abs(Rdot) );
                
                RHS3 = @(Rdot) Rdot * (1 - eta_ratio(i)) * ...
                    geometry(i) * ( ...
                    1 + (dimgroup(i) * abs(Rdot) * geometry(i))^apower ...
                    )^viscositypower;
                
                RHS4 = @(Rdot) -Rdot * (1 - eta_ratio(i)) * ...
                    geometry(i) * ...
                    CY_2F1( obj, dimgroup(i) * abs(Rdot) * geometry(i) );
                
                % This root finding handle is made by combining the RHS
                % handles and subtracting the LHS varuable. 
                handle = @(Rdot) RHS1(Rdot) + RHS2(Rdot) + ...
                    RHS3(Rdot) + RHS4(Rdot) - LHS(i);
                
                % The initial guess for the root finding is the
                % prespecified value, which is the Newtonian case result.
                % Root finding can often go wrong, so  return zero bubble
                % growth in the event of an error, such as non-convergence.
                try
                    dR_dt(1,i) = fzero( handle, LHS(i) );
                catch
                    dR_dt(1,i) = 0;
                end
                
            end % case iteration
            
        end % bubblegrowth
        
    end % methods
    
    %%
    methods (Access = private)
        %%
        % This method can only be accessed from within this class.
        function [ f ] = CY_2F1( obj, y )
            % This function gives the value of Gauss' hypergeometric
            % function:
            %
            %   f = 2F1( 1/a , (1-n)/a ; 1+1/a ; -y^a)
            %
            % 	y       nondimensional strain rate magnitude (scalar!)
            %   a       transition sharpness parameter
            %   n       shear thinning parameter
            %   f       result, a number in (0,1)
            %
            % For derivations of the transformations and equations used,
            % see:
            %
            % "Computation of Hypergeometric Functions, John Pearson,
            % University of Oxford, Mathematical Institute, 2009, MSc in
            % Mathematical Modelling and Scientific Computing"
            %
            
            %%
            % Confirm that the argument y is scalar
            if length(y)~=1
                error('CY_2F1 was called with a non-scalar argument')
            end
            
            % Tolerance toward answer, for infinite sums.
            tol = 10^(-9);
            
            % Calling the properties of the class is slow, so perform it
            % only once for each of a and n to speed up calculation.
            a_val = obj.a;
            n_val = obj.n;
            
            %%
            if y < 0
                %%
                % *********************************************************
                % Negative argument
                % *********************************************************
                
                % The derivation of this function and the transformations
                % used in them only allows positive y, so exit with error
                % if negative y is found.
                
                error('CY_2F1 was called with a negative argument')
                
                
            elseif y <= 1
                %%
                % *********************************************************
                % First Transformation, for y in (0,1)
                % *********************************************************
                
                % Parameter groups, gamma functions, and y terms which are
                % commonly used are precalculated for speed. Referring to
                % properties of the class is also sped up by this, if
                % called many many times.  
                p1 = n_val / a_val;
                p2 = 1 / a_val;
                y1 = ( 1 + y^(-a_val) );
                
                % Start semi-infinite sum at term number k = 0.
                k = 0;
                sum_A = 0;
                A_k = y1^(-p2);
                
                % Keeps going until the next term is smaller than the
                % tolerance.
                while abs(A_k) > tol
                    
                    % Add the previous term, increment term number.
                    sum_A = sum_A + A_k;
                    k = k + 1;
                    
                    % Hypergeometric, so each term is propertional to the
                    % one before by a term that depends on the term number
                    % (k). 
                    A_k = A_k * ...
                        (1 + p1 / k) * (1 - 1 / ( p2 + k ) ) / y1;
                    
                end
                
                % Add the final term calculated, which is smaller than the
                % tolerance, and output. 
                f = sum_A + A_k;
                
                
            else
                %%
                % *********************************************************
                % Second Transformation, for y in (1,infinity)
                % *********************************************************
                
                % Parameter groups, gamma functions, and y terms which are
                % commonly used are precalculated for speed. Referring to
                % properties of the class is also sped up by this, if
                % called many many times. 
                p3 = 1 / a_val - 1;
                p4 = n_val / a_val;
                y2 = ( 1 + y^(a_val) );
                
                % This time there are two semi-infinite summations.
                k=0;
                sum_B = 0;
                B_k = gamma( -p4 ) * gamma( 1 / a_val ) / ...
                    (y2^( 1 / a_val ) * a_val * ...
                    gamma( ( 1 - n_val ) / a_val ));
                sum_C = 0;
                C_k = y2^( ( 1 - n_val ) / a_val ) / n_val;
                
                % Perform each series summation separately, until the
                % tolerance is reached for each. First series: 
                while abs(B_k) > tol
                    
                    % Add the previous term, increment term number.
                    sum_B = sum_B + B_k;
                    k = k + 1;
                    
                    % Hypergeometric multiplier.
                    B_k = B_k * (1 + p3 / k) / y2;
                end
                
                % Add the final term calculated, which is smaller than the
                % tolerance. 
                sum_B = sum_B + B_k;
                
                % Second series, reset the term counter.
                k=0;
                while abs(C_k) > tol
                    
                    % Add the previous term, increment term number.
                    sum_C = sum_C + C_k;
                    k = k + 1;
                    
                    % Hypergeometric multiplier.
                    C_k = C_k * ( 1 + p3 / (k - p4) ) / y2;
                end
                
                % Add each summation, their final terms, which are smaller
                % than the tolerance, and output.
                f = sum_B + B_k + sum_C + C_k;
                
            end % cases of y
            
        end % CY_2F1
        
    end % private methods
    
end