classdef (Abstract) fluid_model
% Copyright Michael Sargent 2015
% 
% FLUID_MODEL is an abstract class which describes a fluid model for use in 
% the three part model for bubble growth. Individual fluid models
% (Newtonian, Powerlaw, Bingham, etc.) are given their own classes as they
% have different functions. These sub-classes must contain at least these
% abstract properties and methods.
% 
%   PROPERTIES
% 
% name                      The name of the fluid model. (string)
% 
% constitutive_equation     The constitutive equation of the fluid model. 
%                           (string)
% 
% -                         Other properties which are specific to each
%                           individual model are specified in the
%                           respective concrete class definitions.
% 
% 
%   METHODS
% 
% viscosity                 Return the viscosity for a given strain rate
%                           and temperature.
% 
% dissipation               Return the rate of viscous dissipation for a
%                           given strain rate and temperature.
% 
% bubblegrowth              Return the bubble growth rate for a given 
%                           pressure difference, bubble size, influence
%                           cell size, and average fluid temperature.
% 
% -                         Set/get methods for the properties are not
%                           specified for the abstract class. The same
%                           applies to the constructor and disp methods.
% 
    
    properties (Abstract, SetAccess=private)
        name@char
        constitutive_equation@char
    end
    
    methods (Abstract)
        eta = viscosity(obj,strainrate,temp)
        phi = dissipation(obj,strainrate,temp)
        Rdot = bubblegrowth(obj,pressurediff,bubblerad,cellrad,temp)
    end
    
end

