function bubble_analyse( savename, good_extra, good_remove, csvname )
%BUBBLE_ANALYSE takes the output file from IDENTIFY_BUBBLES and aligns it
%with the pressure and temperature data recorded by the MPR in a csv file. 
%
%   INPUTS
%
%   savename            string      savename used in identify_bubbles.m to
%                                   create a .mat file
%
%   good_extra          vector      indices of extra bubbles in this track
%                                   to be analysed
%
%   good_remove         vector      indices of bubbles in this track that 
%                                   should not be analysed
%
%   csvname             string      .csv file output from MPR
% 
%
%   OUTPUTS
%
%   none(saved)
%
%%

% load the data
load([savename '_analysis']) 

%%
% track metadata
track_hz = 112.66;
track_time = (1:framenumber)'/track_hz;
track_pix = 1.050; % microns per pixel

% Create the subset of bubbles automatically generated as "good".
good_bool = and(smooth_bool,and(round_bool,noncoalescing_bool));
good_indices = find(good_bool);

% add in the extra manually checked good bubbles, and remove the bad
% bubbles.
checked_indices = union(good_indices,good_extra);
checked_indices(ismember(checked_indices,good_remove)) = [];
checkednumber = length(checked_indices);

% subset the data for checked bubbles
area = area(:,checked_indices)*track_pix^2;
major = major(:,checked_indices)*track_pix;
minor = minor(:,checked_indices)*track_pix;
diameter = diameter(:,checked_indices)*track_pix;

clear smooth_bool round_bool noncoalescing_bool good_bool good_indices
clear checked_indices track_hz track_pix framenumber

%%

% MPR logging of temperature and pressure is in a csv file, one for each
% run.

pressure_hz         = dlmread(csvname,':',[1,1,1,1]);
data_body           = dlmread(csvname,',',30,0);
pressure_time       = data_body(:,1);
pressure            = data_body(:,6);
temperature         = data_body(:,10);

clear csvname data_body csv_start

%%

% match the video track data to the pressure profile

figure('OuterPosition',[1281,1,1280,1024]);

% plot track data, select the start of motion
plot(track_time,diameter)
xlabel('select time for diameters')
set(gca,'xlim',[0 5])
[track_event_time,~]=ginput(1);

% plot the pressure data, select the start of motion
plot(pressure_time,pressure)
xlabel('select time for pressure')
set(gca,'xlim',[0 5])
[pressure_event_time,~]=ginput(1);

close gcf

%%
% modify pressure time so events are coincidental
pressure_time = pressure_time+track_event_time-pressure_event_time;
pressure = pressure*10^5;

% clip or pad the start of pressure_time to 0s
if pressure_time(1) < 0
    pressure = pressure(pressure_time>=0);
    pressure_time = pressure_time(pressure_time>=0);
else
    pressure = [...
       repmat(pressure(1),length(0:(1/pressure_hz):pressure_time(1)),1);...
       pressure(2:end)];
    pressure_time = [...
        (0:(1/pressure_hz):pressure_time(1))';...
        pressure_time(2:end)];
end

% clip or pad the end of pressure_time to 20s
if pressure_time(end) > 20
    pressure = pressure(pressure_time<=20);
    pressure_time = pressure_time(pressure_time<=20);
else
    pressure = [...
        pressure(1:end-1);repmat(pressure(end),...
        length(pressure_time(end):(1/pressure_hz):20),1)];
    pressure_time = [...
        pressure_time(1:end-1);...
        (pressure_time(end):(1/pressure_hz):20)'];
end

% average starting radii and mean temperature
R0 = mean(diameter(track_time<track_event_time,:))/2;
T = mean(temperature);

%%

% show a figure to double check
figure
set(gcf,'OuterPosition',[1281,1,1280,1024]);

subplot(2,1,1)
plot(track_time,diameter)
ylabel('equivalent diameter [\mu{m}]')
xlim([0 20])

subplot(2,1,2)
hold on
plot(...
    pressure_time,...
    pressure,'b-')

xlabel('time [s]')
xlim([0 20])
ylabel('pressure [Pa]')


%%

save([savename '_validation'],...
    'R0','T','area','checkednumber','diameter','major','minor',...
    'pressure','pressure_time','temperature','track_time')


end

