function bubbles = bubble_identify( leveled, thresh )
%BUBBLE_IDENTIFY takes a matrix of a particular seqence, and identifies
%each distinct bubble for every frame. It does not generate the colourised
%matrix.
%   INPUTS
% 
%   leveled             matrix(f,x,y)       f is number of frames, x and y
%                       uint8 or uint16     are resolution, already leveled
%                                           and noise filtered. 
% 
%   thresh              scalar (0-1)        threshold in the levelled
%                                           image, for what is a bubble.
%                                           No argument or -1 is
%                                           autolevelling, 0 is no
%                                           levelling, scalar is a
%                                           specified level.
%
%   OUTPUTS
% 
%   bubbles             structure(f)        one entry per frame
%   -properties         structure(b)        one entry per bubble for each
%                                           frame
% 

%%
% sanitise inputs
if ~isa(leveled,'uint8') && ~isa(leveled,'uint16')
    error('incorrect image format')
end

%%
framenumber = size(leveled,1);

% disp('extracting background level')
h = waitbar(0,'identifying bubbles in leveled image');

%%
%%
tic
% Iterate through the frames.
for i=1:framenumber
    %%
    % Current frame.
    current_frame = squeeze(leveled(i,:,:));
    
    % Threshold and convert to binary where bubbles are true.
    if thresh==-1
        thresh_level = graythresh(current_frame);
    else
        thresh_level = thresh;
    end
    
    current_binary = ~im2bw(current_frame,thresh_level);
    
    % Detect all contiguous objects with 4-connectivity (not diagonal).
    contig_obj = bwconncomp(current_binary,4);
    
    % remove all contiguous objects of below this size (in pixels).
    thresh_size = 20;
    obj_size = cellfun(@length,[contig_obj.PixelIdxList]);
    small_objects = obj_size<=thresh_size;
    contig_obj.PixelIdxList(small_objects)=[];
    contig_obj.NumObjects = length(contig_obj.PixelIdxList);
    
    % convert the object structure to an RGB image, and calculate
    % properties of each object.
    bubbles(i).properties = regionprops(contig_obj,'Centroid','Area',...
        'MajorAxisLength','MinorAxisLength','EquivDiameter','Perimeter',...
        'FilledArea','PixelList');
    
    % estimate time to finish
    etf = toc*(framenumber/i - 1);

    % disp(['completed frame ' num2str(i) ' of ' num2str(framenumber)])
    waitbar(i/framenumber,h,['identified frame ' num2str(i) ' of '...
        num2str(framenumber) ', estimate ' num2str(etf,'%8.0f')...
        ' s remaining'])
        
end

% disp('bubble identification complete')
waitbar(1,h,'bubble identification complete')

close(h)

end