function identify_bubbles(raw,savename)
%IDENTIFY_BUBBLES takes an video matrix and identifies all the bubbles, and
%their properties, and saves the output as a .mat file.
%
%   INPUTS
%
%   raw                 matrix(f,y,x)       f is number of frames, y is
%                                           yresolution, x is xresolution.
%
%   savename            string              name under which to save the
%                                           resulting .mat file
% 
%
%   OUTPUTS
%
%   none(saved)
%

%%
% metadata on the matrix
framenumber = size(raw,1);

%%
% threshhold the raw data, then use it to create the colourised matrix and
% the bubble properties structure
thresh = find_thresh(raw);
colourised = bubble_colourise(raw,thresh);
bubbles = bubble_identify(raw,thresh);

%%
% Track the bubbles forward
% number of bubbles is given by the number in the first frame
bubblenumber = length(bubbles(1).properties);

% preallocate the size of the structure and the fields it contains
all_tracked(bubblenumber).index=[];
all_tracked(bubblenumber).x=[];
all_tracked(bubblenumber).y=[];

h = waitbar(0,'tracking bubbles between frames');
tic
% Iterate throuh all bubbles, extracting data from the first frame
for i=1:length(bubbles(1).properties)
    
    % Keep the index of bubbles in the first frame to identify the bubble
    % through all frames.
    all_tracked(i).index(1) = i;
    
    % Auxiliary data for the bubble in each frame.
    all_tracked(i).x(1) = bubbles(1).properties(i).Centroid(1);
    all_tracked(i).y(1) = bubbles(1).properties(i).Centroid(2);

    % Loop through all remaining frames
    for j=2:framenumber
        
        % Coordinates of the bubble in the previous frame.
        x_prev = all_tracked(i).x(j-1);
        y_prev = all_tracked(i).y(j-1);
        
        % Find the bubble with a centroid closest to this bubble. multiple
        % structure elements causes this strange syntax.
        centers = [bubbles(j).properties.Centroid];
        xs  = centers(1:2:end-1);
        ys = centers(2:2:end);
        dist = sqrt( ( xs - x_prev ).^2 + ( ys - y_prev ).^2 );
        
        % Find the index in the jth frame for this bubble.
        [~,ind] = min(dist);
        all_tracked(i).index(j) = ind;
        
        % Auxiliary data for the bubble in this frame.
        all_tracked(i).x(j) = bubbles(j).properties(ind).Centroid(1);
        all_tracked(i).y(j) = bubbles(j).properties(ind).Centroid(2);
                
    end
    
    % estimate time to finish
    etf = toc*(length(bubbles(1).properties)/i - 1);
    
    % update waitbar
    waitbar(i/length(bubbles(1).properties),h,...
        ['completed bubble ' num2str(i)...
        ' of ' num2str(length(bubbles(1).properties))...
        ', estimate ' num2str(etf,'%8.0f') ' s remaining'])
    
end

close(h)

clear i j x_prev y_prev centers xs ys dist ind h etf

%%
% Check for bubbles which coalesce, by multiple bubbles having the same 
% index in the final frame

% Indices in the final frame of the bubbles in the first frame. Syntax is
% terrible for structures.
final_indices = [all_tracked.index];
final_indices = final_indices(framenumber:framenumber:end);

% Unique final indices. final2unique denotes which elements of
% final_indices appear in unique_indices (in the indexing of
% final_indices), and unique2final denotes in which order the elements of
% unique_indices appear in final_indices (in the indexing of
% unique_indices).  
[unique_indices,final2unique,unique2final] = unique(final_indices);

% Singular coordinates only appear once in final_indices, so are those
% with only one use in unique2final. 
singly_used_unique_indices = ...
    histc(unique2final,1:length(unique_indices))==1;  

% Use final2unique to convert back from the indexing of unique_indices to
% the indexing of final_indices. 
noncoalescing_indices = final2unique(singly_used_unique_indices);

% Turn into a boolean vector
noncoalescing_bool = false(size(bubbles(1).properties));
noncoalescing_bool(noncoalescing_indices) = true;

clear final_indices unique_indices final2unique unique2final 
clear singly_used_unique_indices noncoalescing_indices


%%
% Check for bubbles with outlying step length for the forward direction.
smooth_bool = true(size(bubbles(1).properties));

for i=1:bubblenumber
    x = all_tracked(i).x;
    y = all_tracked(i).y;
    
    dx = diff(x);
    dy = diff(y);    
    
    dr = sqrt( dx.^2 + dy.^2 );
    
    if max(dr)>=5*median(dr)
        smooth_bool(i) = false;
    end
    
end

clear i x y dx dy dr


%%
% Check for wildly aspherical bubbles by area-perimeter ratio
round_bool = true(size(bubbles(1).properties));

for i=1:bubblenumber
    perimeter = bubbles(1).properties(i).Perimeter;
    area = bubbles(1).properties(i).FilledArea;
    
    apr = 4*pi*area/perimeter^2; % 1 for a circle, less for non circular
    
    if apr <0.6
        round_bool(i) = false;
    end
    
end

clear i perimeter area apr


%%

% show colourised matrix's first frame
figure
hold on
imshow(squeeze(colourised(1,:,:,:)))
set(gcf,'OuterPosition',[1281,1,1280,1024]);

% plot the tracks over the colourised frame
hold on
col = (1-[smooth_bool,noncoalescing_bool,round_bool])*0.75;
for i=1:bubblenumber
    plot(all_tracked(i).x,all_tracked(i).y,'color',col(i,:))
    text(all_tracked(i).x(1)+3,all_tracked(i).y(1)+3,...
        cellstr(num2str(i)),'color',col(i,:))
end
title(['Perfect '...
    '{\color{red!50!black} Jumps} '...
    '{\color{green!05!black} Coalesces} '...
    '{\color{blue!50!black} Noncircular}'])
xlabel(['{\color{white!50!black} All fails} '...
    '{\color{yellow!50!black} Jumps and Coalesces} '...
    '{\color{cyan!50!black} Coalesces and Noncircular} '...
    '{\color{magenta!50!black} Jumps and Noncircular}'])
clear i col


%%
% Create a matrix structure to contain all tracked bubble data over all
% frames.

% Create the empty structure of the right size.
bubbles_track(1,2) = bubbles(1).properties(1);
bubbles_track = repmat(bubbles_track(1,1),...
    framenumber,bubblenumber+1);

%%
% Iterate through the frames and then bubbles
h = waitbar(0,'tracking bubbles between frames');
tic
for i=1:framenumber
    
    for j=1:bubblenumber
        
        % Find the index of this bubble in this frame.
        frame_index = all_tracked(j).index(i);
        % Import from the frame the correct bubble.
        bubbles_track(i,j+1) = bubbles(i).properties(frame_index);
    end
    % Concantenate the pixel list only to the first column as 'all
    % bubbles'.
    bubbles_track(i,1).PixelList = [];
    
    % estimate time to finish
    etf = toc*(framenumber/i - 1);
    
    % update waitbar
    waitbar(i/framenumber,h,['completed frame ' num2str(i)...
        ' of ' num2str(framenumber)  ', estimate ' num2str(etf,'%8.0f')...
        ' s remaining'])
end

close(h)

clear i j frame_index h etf

%%

% collate data into vectors 
area = zeros(framenumber,bubblenumber);
major = zeros(framenumber,bubblenumber);
minor = zeros(framenumber,bubblenumber);
diameter = zeros(framenumber,bubblenumber);

for i=1:bubblenumber
    area(:,i) = [bubbles_track(:,i+1).Area]';
    major(:,i) = [bubbles_track(:,i+1).MajorAxisLength]';
    minor(:,i) = [bubbles_track(:,i+1).MinorAxisLength]';
    diameter(:,i) = [bubbles_track(:,i+1).EquivDiameter]';
end
clear i

%%
% save the figure
savefig([savename '_fig'])

% save the data
save([savename '_analysis']) 

end
