function thresh = find_thresh(raw)
%FIND_THRESH is a GUI to find the appropriate threshhold level for a
%sequence of images.
%
%   INPUTS
%
%   raw                 matrix(f,y,x)       f is number of frames, y is
%                                           yresolution, x is xresolution.
% 
%
%   OUTPUTS
%
%   thresh              scalar              threshold selected
%

%%
% initial guess at the threshold
thresh = graythresh(raw);

%%
% first and last frames to check the threshhold against
I = squeeze(raw(1,:,:));
J = squeeze(raw(end,:,:));
h.fig = figure(...
    'OuterPosition',[1281,1,1280,1024]);

%%
% plot both images with their threshholded equivalents
hold on
h.fig1 = subplot(2,2,1);
imshow(I)
h.fig2 = subplot(2,2,2);
imshow(im2bw(I,thresh))
h.fig3 = subplot(2,2,3);
imshow(J)
h.fig4 = subplot(2,2,4);
imshow(im2bw(J,thresh))

%%
% GUI controls

% show the current threshhold
h.thresh = uicontrol(...
    'style','edit',...
    'units','pixels',...
    'position',[820,10,100,35],...
    'string',num2str(thresh),...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@changethresh);

% slider to control threshhold
h.slider = uicontrol(...
    'style','slider',...
    'position',[290,10,500,35],...
    'value',thresh,...
    'min',0,...
    'max',1,...
    'Callback',@changeslider);

% submit button, finished GUI
h.submit = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[950,10,100,35],...
    'string','submit',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@submit);

% wait until something is changed
uiwait(h.fig)

%%
% callback functions for the GUI controls

    function changethresh(hObj,event)
        % Called when the textbox for threshold is changed.
        
        % get string from UI, convert to number
        string = get(hObj,'string');
        thresh = str2double(string);
        
        % sanitise input
        if thresh<=0
            thresh = 0;
        elseif thresh>=1
            thresh = 1;
        end
        
        % change displayed images
        subplot(h.fig2)
        imshow(im2bw(I,thresh))
        subplot(h.fig4)
        imshow(im2bw(J,thresh))
        
        % change value of the slider
        set(h.slider,'value',thresh)
        
    end

    function changeslider(hObj,event)
        % Called when the slider position changes
        
        % get value from slider
        thresh = get(hObj,'value');
        
        % change displayed images
        subplot(h.fig2)
        imshow(im2bw(I,thresh))
        subplot(h.fig4)
        imshow(im2bw(J,thresh))
        
        % change textbox number
        set(h.thresh,'string',num2str(thresh))
        
    end

    function submit(hObj,event)
        % Called when the submit button is pressed
        uiresume(h.fig)
        
        % closes the figure
        close(h.fig)
        
    end

end