function matrix_viewer( matrix )
%MATRIX_VIEWER provides a GUI with which to view a sequence of images. It
%may be used in raw or colourised data.

%%
% Get the number of frames and the resolution from the bayer images.
framenumber = size(matrix,1);
yresolution = size(matrix,2);
xresolution = size(matrix,3);
cresolution = size(matrix,4);

% check the colour mode of the matrix
if cresolution==3
    % RGB
elseif cresolution==1
    % monochrome to RGB monochrome
    matrix = repmat(matrix,1,1,1,3);
else
    error('unknown number of colour dimensions')
end

%%
% Boundary sizes for possible image placement.
border = 80;
boundsx = 1120;
boundsy = 810;
boundsaspect = boundsx / boundsy;

% Display method depends on apect ratio.
imageaspect = xresolution / yresolution;

% Image width and height will usually only have one dimension hit the
% bounds, the other will be less than the bounds. 
if imageaspect>=boundsaspect
    imagex = boundsx;
    zoom = boundsx / xresolution;
    imagey = floor( yresolution * zoom );
    yskip = floor( ( boundsy - imagey ) / 2 );
    imagelocation = [border,border+yskip,imagex,imagey];
    
else
    imagey = boundsy;
    zoom = boundsy / yresolution;
    imagex = floor( xresolution * zoom );
    xskip = floor( ( boundsx - imagex ) / 2 );
    imagelocation = [border+xskip,border,imagex,imagey];
    
end

% Current frame index and colour option, initialised to the first frame,
% and Bayer colour.
currentframe = 1;

%%
% Form the GUI, the h variable contains all handles.
h.fig = figure(...
    'OuterPosition',[1281,1,1280,1024]);

% Forward and backward buttons.
h.forward = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[1220,472,40,40],...
    'string','>',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@moveforward);

h.backward = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[20,472,40,40],...
    'string','<',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@movebackward);

% Display the current frame number.
h.frame = uicontrol(...
    'style','edit',...
    'units','pixels',...
    'position',[590,10,100,35],...
    'string','1',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@changeframe);

% Create play and reverse buttons
h.play = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[710,10,35,35],...
    'string','>',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'callback',@playsequence);

h.reverse = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[535,10,35,35],...
    'string','<',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'callback',@reversesequence);

% Create play and reverse buttons
h.fastforward = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[755,10,35,35],...
    'string','>>',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'callback',@fastforwardsequence);

h.fastreverse = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[480,10,35,35],...
    'string','<<',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'callback',@fastreversesequence);

h.axes = axes(...
    'units','pixels',...
    'position',imagelocation);
axis off
axis image

% Display the current frame.
displayframe

% Neither palying nor reversing
playing = false;
reversing = false;

%%
    function moveforward(hObj,event)
        % Called when the forward button is pressed.
        
        % Find new frame number, looping the ends.
        if currentframe ~= framenumber
            currentframe = currentframe + 1;
        else
            currentframe = 1;
        end
        
        % Change the displayed frame.
        displayframe
        
    end

%%
    function movebackward(hObj,event)
        % Called when the backward button is pressed.
        
        % Find new frame number, looping the ends.
        if currentframe ~= 1
            currentframe = currentframe - 1;
        else
            currentframe = framenumber;
        end
        
        % Change the displayed frame.
        displayframe
        
    end

%%
    function playsequence(hObj,event)
        % Called when the play is pressed.
        
        % Set appropriate booleans.
        playing = true;
        reversing = false;
        
        % Change the funtionality of buttons.
        set(h.play,'string','[]','callback',@stopsequence)
        set(h.reverse,'string','<','callback',@reversesequence)
        set(h.fastforward,'string','>>','callback',@fastforwardsequence)
        set(h.fastreverse,'string','<<','callback',@fastreversesequence)
        
        while playing
            
            moveforward([],[])
            
        end
        
    end

%%
    function reversesequence(hObj,event)
        % Called when the reverse is pressed.
        
        % Set appropriate booleans.
        playing = false;
        reversing = true;
        
        % Change the funtionality of buttons.
        set(h.play,'string','>','callback',@playsequence)
        set(h.reverse,'string','[]','callback',@stopsequence)
        set(h.fastforward,'string','>>','callback',@fastforwardsequence)
        set(h.fastreverse,'string','<<','callback',@fastreversesequence)
        
        while reversing
            
            movebackward([],[])
            
        end
        
    end

%%
    function fastforwardsequence(hObj,event)
        % Called when the fastforward is pressed.
        
        % Set appropriate booleans.
        playing = true;
        reversing = false;
        
        % Change the funtionality of buttons.
        set(h.play,'string','>','callback',@playsequence)
        set(h.reverse,'string','<','callback',@reversesequence)
        set(h.fastforward,'string','[]','callback',@stopsequence)
        set(h.fastreverse,'string','<<','callback',@fastreversesequence)
        
        while playing
            
            currentframe = currentframe+4;
            moveforward([],[])
            
        end
        
    end

%%
    function fastreversesequence(hObj,event)
        % Called when the reverse is pressed.
        
        % Set appropriate booleans.
        playing = false;
        reversing = true;
        
        % Change the funtionality of buttons.
        set(h.play,'string','>','callback',@playsequence)
        set(h.reverse,'string','<','callback',@reversesequence)
        set(h.fastforward,'string','>>','callback',@fastforwardsequence)
        set(h.fastreverse,'string','[]','callback',@stopsequence)
        
        while reversing
            
            currentframe = currentframe+4;
            movebackward([],[])
            
        end
        
    end

%%
    function stopsequence(hObj,event)
        % Called when either the play or stop button is pressed while
        % active. 
        
        % set to neither play nor reverse
        playing = false;
        reversing = false;
        
        % set neither button to stop
        set(h.play,'string','>','callback',@playsequence)
        set(h.reverse,'string','<','callback',@reversesequence)
        
    end

%%
    function changeframe(hObj,event)
        % Called when the number is changed.
        
        % stop playing or reversing
        stopsequence
        
        % get string from UI, convert to number
        string = get(hObj,'string');
        newframe = str2double(string);
        
        % force integer
        newframe = floor(newframe);
        
        % non-number strings do nothing
        if newframe>=1 && newframe<=framenumber
            currentframe = newframe;
        end
        
        % display
        displayframe
        
    end

%%
    function displayframe
        % Called whenever the frame is changed.
        
        % Change the displayed frame number.
        set(h.frame,'string',int2str(currentframe))
        
        % change the displayed image
        imagematrix = squeeze(matrix(currentframe,:,:,:));
        axes(h.axes)
        imshow(imagematrix,[])
            
        pause(1/50)
        
    end

end