function track_viewer( bubbles_track, xresolution, yresolution )
%TRACK_VIEWER provides a GUI with which to view a sequence of images that
%has been converted into a bubble track structure by bubble_track.m

%%
% Get the number of frames and bubbles from the size of the track
% structure. 
framenumber = size(bubbles_track,1);
bubblenumber = size(bubbles_track,2);

% Initially do not display bubble numbers.
displaynumbers = false;

% Generate menu list for bubble number, first entry is all bubbles, remove
% brackets, and replace spaces with pipes.
menu = mat2str(1:bubblenumber-1);
menu = regexprep(menu,'[','');
menu = regexprep(menu,']','');
menu = regexprep(menu,' ','|');
menu = ['all|' menu];

%%
% Boundary sizes for possible image placement.
border = 80;
boundsx = 1120;
boundsy = 810;
boundsaspect = boundsx / boundsy;

% Display method depends on apect ratio.
imageaspect = xresolution / yresolution;

% Image width and height will usually only have one dimension hit the
% bounds, the other will be less than the bounds.
if imageaspect>=boundsaspect
    imagex = boundsx;
    zoom = boundsx / xresolution;
    imagey = floor( yresolution * zoom );
    yskip = floor( ( boundsy - imagey ) / 2 );
    imagelocation = [border,border+yskip,imagex,imagey];
    
else
    imagey = boundsy;
    zoom = boundsy / yresolution;
    imagex = floor( xresolution * zoom );
    xskip = floor( ( boundsx - imagex ) / 2 );
    imagelocation = [border+xskip,border,imagex,imagey];
    
end

%%
% Form the GUI, the h variable contains all handles.
h.fig = figure(...
    'OuterPosition',[1281,1,1280,1024]);

% Menu to select the bubble.
h.menu = uicontrol(...
    'style','popupmenu',...
    'units','pixels',...
    'position',[590,910,100,35],...
    'string',menu,...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@setbubble);

% Checkbox to display bubble numbers
h.numbers = uicontrol(...
    'style','togglebutton',...
    'units','pixels',...
    'position',[710,910,300,35],...
    'string','Display bubble numbers',...
    'Value',0,...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@numberscheck);

% Forward and backward buttons.
h.forward = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[1220,472,40,40],...
    'string','>',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@moveforward);

h.backward = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[20,472,40,40],...
    'string','<',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@movebackward);

% Display the current frame number.
h.frame = uicontrol(...
    'style','edit',...
    'units','pixels',...
    'position',[590,10,100,35],...
    'string','1',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'Callback',@changeframe);

% Create play and reverse buttons
h.play = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[710,10,35,35],...
    'string','>',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'callback',@playsequence);

h.reverse = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[535,10,35,35],...
    'string','<',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'callback',@reversesequence);

% Create play and reverse buttons
h.fastforward = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[755,10,35,35],...
    'string','>>',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'callback',@fastforwardsequence);

h.fastreverse = uicontrol(...
    'style','pushbutton',...
    'units','pixels',...
    'position',[480,10,35,35],...
    'string','<<',...
    'fontsize',18,...
    'foregroundcolor','k',...
    'callback',@fastreversesequence);

h.axes = axes(...
    'units','pixels',...
    'position',imagelocation);

axis off
axis image

%%
% Current frame index and bubble selection, initialised to the first frame
% and all bubbles.
currentframe = 1;
currentbubble = 1;

% Variables to tell when the sequence is playing, reversing, or not.
playing = false;
reversing = false;

% Display the current frame.
displayframe

%%
    function setbubble(hObj,event)
        % Called when a new bubble is set in the menu.
        
        % Retrieve the selected menu item.
        currentbubble = get(hObj,'Value');
        
        % Change the displayed frame.
        displayframe
        
    end

%%
    function numberscheck(hObj,event)
        % Called when the checkbox is checked or unchecked.
        
        % Check the status of the box, update the status of the global
        % boolean. 
        displaynumbers = get(hObj,'Value');
        
        % Change the displayed frame.
        displayframe
        
        
    end

%%
    function moveforward(hObj,event)
        % Called when the forward button is pressed.
        
        % Find new frame number, looping the ends.
        if currentframe ~= framenumber
            currentframe = currentframe + 1;
        else
            currentframe = 1;
        end
        
        % Change the displayed frame.
        displayframe
        
    end

%%
    function movebackward(hObj,event)
        % Called when the backward button is pressed.
        
        % Find new frame number, looping the ends.
        if currentframe ~= 1
            currentframe = currentframe - 1;
        else
            currentframe = framenumber;
        end
        
        % Change the displayed frame.
        displayframe
        
    end

%%
    function playsequence(hObj,event)
        % Called when the play is pressed.
        
        % Set appropriate booleans.
        playing = true;
        reversing = false;
        
        % Change the funtionality of buttons.
        set(h.play,'string','[]','callback',@stopsequence)
        set(h.reverse,'string','<','callback',@reversesequence)
        set(h.fastforward,'string','>>','callback',@fastforwardsequence)
        set(h.fastreverse,'string','<<','callback',@fastreversesequence)
        
        while playing
            
            moveforward([],[])
            
        end
        
    end

%%
    function reversesequence(hObj,event)
        % Called when the reverse is pressed.
        
        % Set appropriate booleans.
        playing = false;
        reversing = true;
        
        % Change the funtionality of buttons.
        set(h.play,'string','>','callback',@playsequence)
        set(h.reverse,'string','[]','callback',@stopsequence)
        set(h.fastforward,'string','>>','callback',@fastforwardsequence)
        set(h.fastreverse,'string','<<','callback',@fastreversesequence)
        
        while reversing
            
            movebackward([],[])
            
        end
        
    end

%%
    function fastforwardsequence(hObj,event)
        % Called when the fastforward is pressed.
        
        % Set appropriate booleans.
        playing = true;
        reversing = false;
        
        % Change the funtionality of buttons.
        set(h.play,'string','>','callback',@playsequence)
        set(h.reverse,'string','<','callback',@reversesequence)
        set(h.fastforward,'string','[]','callback',@stopsequence)
        set(h.fastreverse,'string','<<','callback',@fastreversesequence)
        
        while playing
            
            currentframe = currentframe+4;
            moveforward([],[])
            
        end
        
    end

%%
    function fastreversesequence(hObj,event)
        % Called when the reverse is pressed.
        
        % Set appropriate booleans.
        playing = false;
        reversing = true;
        
        % Change the funtionality of buttons.
        set(h.play,'string','>','callback',@playsequence)
        set(h.reverse,'string','<','callback',@reversesequence)
        set(h.fastforward,'string','>>','callback',@fastforwardsequence)
        set(h.fastreverse,'string','[]','callback',@stopsequence)
        
        while reversing
            
            currentframe = currentframe+4;
            movebackward([],[])
            
        end
        
    end

%%
    function stopsequence(hObj,event)
        % Called when either the play or stop button is pressed while
        % active. 
        
        % Set appropriate booleans.
        playing = false;
        reversing = false;
        
        % Change the funtionality of buttons.
        set(h.play,'string','>>','callback',@playsequence)
        set(h.reverse,'string','<<','callback',@reversesequence)
        
    end

%%
    function changeframe(hObj,event)
        % Called when frame number is edited.
        
        % Stop playing or reversing.
        stopsequence
        
        % Get string from UI, convert to number, force as integer
        string = get(hObj,'string');
        newframe = floor(str2double(string));
        
        % Non-number strings return null
        if newframe>=1 && newframe<=framenumber
            currentframe = newframe;
        end
        
        % display
        displayframe
        
    end

%%
    function displayframe
        % Called whenever the frame is changed.
        
        % Change the displayed frame number.
        set(h.frame,'string',int2str(currentframe))
        
        % Extract the image of the current bubble in boolean
        imagematrix = full(logical(sparse(...
            bubbles_track(currentframe,currentbubble).PixelList(:,2),...
            bubbles_track(currentframe,currentbubble).PixelList(:,1),...
            1,...
            yresolution,...
            xresolution)));
        
        % Show the image on the axes
        axes(h.axes)
        imshow(imagematrix,[])
            
        pause(1/50)
        
        % display numbers if box is checked
        if displaynumbers
            for i=2:bubblenumber
                text(bubbles_track(currentframe,i).Centroid(1),...
                    bubbles_track(currentframe,i).Centroid(2),...
                    num2str(i-1),...
                    'HorizontalAlignment','center',...
                    'color','r');
            end
        end
        
    end

end